﻿using System;
using System.Linq;
using System.Threading.Tasks;
using Discord;
using Discord.Commands;
using Geekbot.net.Database;
using Geekbot.net.Database.Models;
using Geekbot.net.Lib;
using Geekbot.net.Lib.CommandPreconditions;
using Geekbot.net.Lib.ErrorHandling;
using Geekbot.net.Lib.Extensions;

namespace Geekbot.net.Commands.User
{
    [DisableInDirectMessage]
    public class Karma : GeekbotBase
    {
        private readonly IErrorHandler _errorHandler;
        private readonly DatabaseContext _database;

        public Karma(DatabaseContext database, IErrorHandler errorHandler)
        {
            _database = database;
            _errorHandler = errorHandler;
        }

        [Command("good", RunMode = RunMode.Async)]
        [Summary("Increase Someones Karma")]
        public async Task Good([Summary("@someone")] IUser user)
        {
            try
            {
                var actor = await GetUser(Context.User.Id);
                if (user.Id == Context.User.Id)
                {
                    await ReplyAsync(Context.Translations.GetString("CannotChangeOwn", Context.User.Username));
                }
                else if (TimeoutFinished(actor.TimeOut))
                {
                    var formatedWaitTime = Context.Translations.FormatDateTimeAsRemaining(actor.TimeOut.AddMinutes(3));
                    await ReplyAsync(Context.Translations.GetString("WaitUntill", Context.User.Username, formatedWaitTime));
                }
                else
                {
                    var target = await GetUser(user.Id);
                    target.Karma = target.Karma + 1;
                    SetUser(target);
                    
                    actor.TimeOut = DateTimeOffset.Now;
                    SetUser(actor);

                    await _database.SaveChangesAsync();

                    var eb = new EmbedBuilder();
                    eb.WithAuthor(new EmbedAuthorBuilder()
                        .WithIconUrl(user.GetAvatarUrl())
                        .WithName(user.Username));

                    eb.WithColor(new Color(138, 219, 146));
                    eb.Title = Context.Translations.GetString("Increased");
                    eb.AddInlineField(Context.Translations.GetString("By"), Context.User.Username);
                    eb.AddInlineField(Context.Translations.GetString("Amount"), "+1");
                    eb.AddInlineField(Context.Translations.GetString("Current"), target.Karma);
                    await ReplyAsync("", false, eb.Build());
                }
            }
            catch (Exception e)
            {
                await _errorHandler.HandleCommandException(e, Context);
            }
        }

        [Command("bad", RunMode = RunMode.Async)]
        [Summary("Decrease Someones Karma")]
        public async Task Bad([Summary("@someone")] IUser user)
        {
            try
            {
                var actor = await GetUser(Context.User.Id);
                if (user.Id == Context.User.Id)
                {
                    await ReplyAsync(Context.Translations.GetString("CannotChangeOwn", Context.User.Username));
                }
                else if (TimeoutFinished(actor.TimeOut))
                {
                    var formatedWaitTime = Context.Translations.FormatDateTimeAsRemaining(actor.TimeOut.AddMinutes(3));
                    await ReplyAsync(Context.Translations.GetString("WaitUntill", Context.User.Username, formatedWaitTime));
                }
                else
                {
                    var target = await GetUser(user.Id);
                    target.Karma = target.Karma - 1;
                    SetUser(target);
                    
                    actor.TimeOut = DateTimeOffset.Now;
                    SetUser(actor);

                    await _database.SaveChangesAsync();

                    var eb = new EmbedBuilder();
                    eb.WithAuthor(new EmbedAuthorBuilder()
                        .WithIconUrl(user.GetAvatarUrl())
                        .WithName(user.Username));

                    eb.WithColor(new Color(138, 219, 146));
                    eb.Title = Context.Translations.GetString("Decreased");
                    eb.AddInlineField(Context.Translations.GetString("By"), Context.User.Username);
                    eb.AddInlineField(Context.Translations.GetString("Amount"), "-1");
                    eb.AddInlineField(Context.Translations.GetString("Current"), target.Karma);
                    await ReplyAsync("", false, eb.Build());
                }
            }
            catch (Exception e)
            {
                await _errorHandler.HandleCommandException(e, Context);
            }
        }

        private bool TimeoutFinished(DateTimeOffset lastKarma)
        {
            return lastKarma.AddMinutes(3) > DateTimeOffset.Now;
        }

        private async Task<KarmaModel> GetUser(ulong userId)
        {
            var user = _database.Karma.FirstOrDefault(u =>u.GuildId.Equals(Context.Guild.Id.AsLong()) && u.UserId.Equals(userId.AsLong())) ?? await CreateNewRow(userId);
            return user;
        }
        
        private void SetUser(KarmaModel user)
        {
            _database.Karma.Update(user);
        }
        
        private async Task<KarmaModel> CreateNewRow(ulong userId)
        {
            var user = new KarmaModel()
            {
                GuildId = Context.Guild.Id.AsLong(),
                UserId = userId.AsLong(),
                Karma = 0,
                TimeOut = DateTimeOffset.MinValue
            };
            var newUser = _database.Karma.Add(user).Entity;
            await _database.SaveChangesAsync();
            return newUser;
        }
    }
}